(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
(*  Soubor: KLAVESY.PAS                                                    *)
(*  Obsah: kody klaves, procedury pro cteni z klavesnice a jeji nastaveni  *)
(*  Autor: Mircosoft (http://mircosoft.mzf.cz)                             *)
(*         SWAG team (DOSova varianta Keypressed a Readkey)                *)
(*         Ronny Wester (cast Asm preruseni klavesnice)                    *)
(*  Posledni uprava: 22.3.2008                                             *)
(*  Pro kompilaci: (DOS.TPU)                                               *)
(*  Pro spusteni: nic                                                      *)
(*  Upozorneni: tyto zdrojove kody pouzivate na vlastni nebezpeci          *)
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
unit klavesy;

interface

(***************** standardni funkce pro cteni klaves: **********************)

function KeyPressed:boolean;
function ReadKey:char;
{Funguji uplne stejne jako standardni z jednotky CRT.
Pomoci nasledujicich symbolu si muzete vybrat, jakym zpusobem se maji tyto
funkce interpretovat (jeden symbol nechte definovany, ostatni nedefinovane):}
    {$define readkey_DOS}
{Spolehliva a odzkousena varianta cteni klavesnice pres int $21 (sluzby DOSu).
Readkey pri stisku rozsirene klavesy vraci postupne dva kody: #0 a potom kod
klavesy (totez co ten z CRT). Vicemene jedina prakticky pouzitelna varianta.}
    {...$define readkey_nerozsirena}
{Varianta pro cteni pres int $16 (sluzby klavesnice). Pri stisku rozsirene
klavesy vraci Readkey pouze znak #0 (a pak uz nic).}
    {...$define readkey_rozsirena}
{Varianta pro cteni pres int $16, jen pro rozsirenou klavesnici (tj. prakticky
vsechny, s jednotkou Enhkbd to nema nic spolecneho). Pri stisku rozsirene
klavesy vraci Readkey pouze znak #$E0 ('').}
    {...$define readkey_122klaves}
{Varianta pro cteni pres int $16, ale jen pro 122klavesove klavesnice, ktere
jsou dost vzacne (tj. prakticky k nicemu).}

{kody nekterych rozsirenych klaves
(dosovy Readkey je vraci jako #0 a pak #tohle):}
const
F1=59;     shiftF1=84;     ctrlF1=94;      altF1=104;
F2=60;     shiftF2=85;     ctrlF2=95;      altF2=105;
F3=61;     shiftF3=86;     ctrlF3=96;      altF3=106;
F4=62;     shiftF4=87;     ctrlF4=97;      altF4=107;
F5=63;     shiftF5=88;     ctrlF5=98;      altF5=108;
F6=64;     shiftF6=89;     ctrlF6=99;      altF6=109;
F7=65;     shiftF7=90;     ctrlF7=100;     altF7=110;
F8=66;     shiftF8=91;     ctrlF8=101;     altF8=111;
F9=67;     shiftF9=92;     ctrlF9=102;     altF9=112;
F10=68;    shiftF10=93;    ctrlF10=103;    altF10=113;
F11=133;   shiftF11=135;   ctrlF11=137;    altF11=139;
F12=134;   shiftF12=136;   ctrlF12=138;    altF12=140;
lsipka=75; {75}            ctrllsipka=115; altlsipka=155;
psipka=77; {77}            ctrlpsipka=116; altpsipka=157;
hsipka=72; shiftHsipka=72; ctrlhsipka=141; althsipka=152;
dsipka=80; shiftDsipka=80; ctrldsipka=145; altdsipka=160;
ins=82;    {82}            ctrlins=146;    altins=162;
del=83;    {83}            ctrldel=147;    altdel=163;
home=71;   {71}            ctrlhome=119;   althome=151;
endk=79;   {79}            ctrlend=117;    altend=159;
pgup=73;   {73}            ctrlpgup=132;   altpgup=153;
pgdn=81;   {81}            ctrlpgdn=118;   altpgdn=161;
{F11, F12, Alt+sipky, Ctrl+sipka nahoru nebo dolu a nektere dalsi kombinace
funguji jen pri pouziti jednotky EnhKbd.}


(********* dalsi procedury a funkce pro standardni cteni klaves: ************)

function xReadKey:word;
{Vraci ordinalni cislo stisknute klavesy. U rozsirenych klaves (ty, co vraceji
nejdriv #0 a az pak nejake cislo) vraci to druhe cislo zvetsene o 256 (takze
odpada druhe cteni jako u Readkey). Pouziva int $21.}

{kody klaves, jak je vraci funkce xReadkey:}
const
xF1=315;     xShiftF1=340;     xCtrlF1=350;      xAltF1=360;
xF2=316;     xShiftF2=341;     xCtrlF2=351;      xAltF2=361;
xF3=317;     xShiftF3=342;     xCtrlF3=352;      xAltF3=362;
xF4=318;     xShiftF4=343;     xCtrlF4=353;      xAltF4=363;
xF5=319;     xShiftF5=344;     xCtrlF5=354;      xAltF5=364;
xF6=320;     xShiftF6=345;     xCtrlF6=355;      xAltF6=365;
xF7=321;     xShiftF7=346;     xCtrlF7=356;      xAltF7=366;
xF8=322;     xShiftF8=347;     xCtrlF8=357;      xAltF8=367;
xF9=323;     xShiftF9=348;     xCtrlF9=358;      xAltF9=368;
xF10=324;    xShiftF10=349;    xCtrlF10=359;     xAltF10=369;
xF11=484;    xShiftF11=251;    xCtrlF11=393;     xAltF11=395;
xF12=352;    xShiftF12=80;     xCtrlF12=370;     xAltF12=396;
xLSipka=331; xShiftLSipka=331; xCtrlLSipka=371;  xAltLsipka=411;
xPSipka=333; xShiftPSipka=333; xCtrlPSipka=372;  xAltPSipka=413;
xHSipka=328; xShiftHSipka=328; xCtrlHSipka=397;  xAltHSipka=408;
xDSipka=336; xShiftDSipka=336; xCtrlDSipka=401;  xAltDSipka=416;
xIns=338;
xDel=339;
xHome=327;
xEndk=335;
xPgUp=329;
xPgDn=337;
xCtrlG=7; {ctrl+G}
{Pismena a cislice jsou jejich bezna ordinalni cisla, jenom ve formatu word.
Nektere kombinace klaves opet funguji jen s jednotkou EnhKbd - viz vyse.}

procedure kCekej;
{cekani na stisk klavesy}
procedure kResetuj;
{vyprazdneni bufferu klavesnice}

procedure ToKeyBuf(s:string);
{Zapise retezec s do bufferu klavesnice, jako kdyby ho tam napsal uzivatel.
Max. 15 znaku - vic se do bufferu nevejde a klavesnice zacne piskat.}
procedure RychlostKlavesnice(Zpozdeni,Rychlost:byte);
{Nastavi rychlost opakovani stisknute klavesy a zpozdeni po prvnim stisku.
mozne hodnoty parametru Zpozdeni:
 0 = 250 ms
 1 = 500 ms
 2 = 750 ms
 3 = 1000 ms
a parametru Rychlost:
  0 = 30  zn/s
  8 = 15  zn/s
 16 = 7.5 zn/s
 24 = 3.7 zn/s
 31 = 2   zn/s}
function KlavesniceUmi(co:byte):boolean;
{Zjisti, jestli klavesnice podporuje danou funkci. Za parametr Co muzete
dosadit nasledujici hodnoty:}
const klNastavitVychoziRychlost=0;
      klVypnoutOpakovani=1;
      klNastavitRychlost=2; {tj. jestli bude fungovat procedura Rychlostklavesnice}
      klZjistitRychlost=3;
      klZjistitTyp=4;
      klJeRozsirena=5; {hodi se toto otestovat drive nez zacneme cekat na stisk F12 apod.}
      klMa122Klaves=6;


(********************* shifty, alty, ctrly a locky: *************************)

function LShiftPressed:boolean;{stisknut levy shift}
function PShiftPressed:boolean;{stisknut pravy shift}
function ShiftPressed:boolean;{stisknut libovolny shift}
function LCtrlPressed:boolean;{stisknut levy control}
function PCtrlPressed:boolean;{stisknut pravy control}
function CtrlPressed:boolean;{stisknut libovolny control}
function LAltPressed:boolean;{stisknut levy alt}
function PAltPressed:boolean;{stisknut pravy alt}
function AltPressed:boolean;{stisknut libovolny alt}
function CapsLockPressed:boolean;{stisknut caps lock}
function NumLockPressed:boolean;{stisknut num lock}
function ScrollLockPressed:boolean;{stisknut scroll lock}
function CapsLock:boolean;{caps lock zapnut}
function NumLock:boolean;{num lock zapnut}
function ScrollLock:boolean;{scroll lock zapnut}
{tyto funkce vraceji true, pokud je splnena prislusna podminka}

procedure SetCapsLock(jak:boolean);
procedure SetNumLock(jak:boolean);
procedure SetScrollLock(jak:boolean);
{tyto procedury zapinaji nebo vypinaji prislusne locky (true=zap., false=vyp.)}


(******************** specialni obsluha klavesnice: *************************)

procedure InitKlav;
{Nainstaluje nove preruseni pro obsluhu klavesnice, takze pujde cist vic
klaves stisknutych soucasne (bez bufferu a bez cekani), ale prestanou fungovat
vsechny vyse uvedene procedury a funkce pro obycejny bufferovany vstup
(Keypressed, Readkey atd), proto je nezkousejte volat! (vyjimkou je cteni
shiftu a spol., ty sice vrati nesmysly, ale aspon nekousnou pocitac)}
procedure ZrusKlav;
{Obnovi puvodni funkci klavesnice. Pokud ji nezavolate rucne, spusti se
automaticky pri ukonceni programu.
Pozor: po obnoveni puvodniho preruseni se klavesnice obcas chova tak, jako by
byl stale stisknuty Ctrl. Nevim, cim to je, staci ale ten Ctrl jednou
zmacknout a pustit a je po problemu.}
function KlavesyInstalovany:boolean;
{Vraci true, pokud je nainstalovana specialni obsluha klavesnice.}

function pKeyPressed:boolean;
{Vraci true, kdyz je neco zmacknuto. Funguje jen pri specialni obsluze
klavesnice, jinak vrati nesmysl.}
function Pressed(klavesa:byte):boolean;
{Rika, jestli je dana klavesa stisknuta. Napr.: if pressed(pEnter) then ...
Opet funguje jen pri specialni obsluze, jinak vraci nesmysly.

Kody klaves pro funkci Pressed:}
const
pEsc=1;        pScrollLock=70{*}; pPomlcka=12;      pOk=24;
pF1=59;        pBreak=29{**};     pRovnitko=13;     pPk=25;
pF2=60;        pVlnovka=41;       pBackSlash=43;    pLHZavorka=26;{ty hranate vedle P}
pF3=61;        p1k=2;             pBackspace=14;    pPHZavorka=27;
pF4=62;        p2k=3;             pTab=15;          pEnter=28;{ten velky}
pF5=63;        p3k=4;             pQk=16;           pCapsLock=58{*};
pF6=64;        p4k=5;             pWk=17;           pAk=30;
pF7=65;        p5k=6;             pEk=18;           pSk=31;
pF8=66;        p6k=7;             pRk=19;           pDk=32;
pF9=67;        p7k=8;             pTk=20;           pFk=33;
pF10=68;       p8k=9;             pYk=21;           pGk=34;
pF11=87;       p9k=10;            pUk=22;           pHk=35;
pF12=88;       p0k=11;            pIk=23;           pJk=36;
pKk=37;        pCarka=51;         pPgDn=209;        pNum1=79;
pLk=38;        pTecka=52;         pLSipka=203;      pNum2=80;
pStrednik=39;  pLomitko=53;       pPSipka=205;      pNum3=81;
pApostrof=40;  pPShift=54;        pHSipka=200;      pNum4=75;
pLShift=42;    pCtrl=29;          pDSipka=208;      pNum5=76;
pZk=44;        pLevyAlt=56;       pNumLock=69{*};   pNum6=77;
pXk=45;        pPravyAlt=184;     pNumHvezdicka=55; pNum7=71;
pCk=46;        pIns=210;          pNumMinus=74;     pNum8=72;
pVk=47;        pDel=211;          pNumPlus=78;      pNum9=73;
pBk=48;        pHome=199;         pNumTecka=83;     pNum0=82;
pNk=49;        pEnd=207;       {+}pPrintScreen=55;  pNumLomitko=181;
pMk=50;        pPgUp=201;         pMezernik=57;     pNumEnter=156;{maly enter na numpadu}
pZapnutyNumlock=170;{++}
{*: Lockum moc neverte, v tomto rezimu jsou pomerne nevyzpytatelne (nekdy dany
    lock nastavi, jindy ne... zalezi na klavesnici). Spolehliva je pouze
    detekce jejich stisku.
 **: Pause/Break (bez Ctrl) na vestine pocitacu zpusobuje standardne
     pozastaveni programu do stisku libovolne klavesy.
 +: Printscreen v textovem rezimu pod Windows nekdy zpusobuje Modrou smrt
    (BSOD).
 ++: Kdyz je zapnuty Numlock, aktivuje se po stisku home, end, insert, delete,
     page up, page down nebo sipek i hodnota 170.

Kody vsech klaves jsou porad stejne, nezalezi na tom, jestli k nim drzite
shift, alt nebo ctrl nebo na stavu jakychkoli locku.}


implementation (*************************************************************)

uses dos;{pro getintvec a setintvec}

{$ifdef readkey_DOS}

Function Keypressed:Boolean; assembler;
Asm
mov AX,$0B00
int $21
{funkce vraci hodnotu z AL; kdyz je nenulova, je to true}
End;{keypressed}

function ReadKey:char; assembler;
Asm
mov AX,$0700
int $21
{funkce vraci hodnotu z AL - znak}
End;{readkey}

procedure kcekej; assembler;
Asm
 @CekejDal:
 mov AX,$0B00
 int $21       {keypressed?}
 or AL,AL
 jz @CekejDal  {kdyz je to nula (false), jdeme zpatky na zacatek}
End;{kcekej}

procedure kresetuj; assembler;
Asm
mov AX,$0C06
mov DL,$FF
int $21
End;{kresetuj}

{$else}

procedure kcekej;
Begin
repeat until keypressed;
End;{kcekej}

procedure kresetuj;
var tmp:char;
Begin
while keypressed do tmp:=readkey;
End;{kresetuj}

{$endif}

{$ifdef readkey_nerozsirena}

function keypressed:boolean; assembler;
Asm
mov AH,1
int $16    {vysledek se vraci v ZF}
mov AX,1   {ZF se timhle neovlivni}
jnz @ano   {jestli je ZF=0 (byla stisknuta klavesa), preskocime vynulovani AX}
 xor AX,AX  {jinak ho nulujeme (=false)}
@ano:
End;{keypressed}

function readkey:char; assembler;
Asm
xor AH,AH
int $16
End;{readkey}

{$endif}

{$ifdef readkey_rozsirena}

function keypressed:boolean; assembler;
Asm
mov AH,$11
int $16
mov AX,1
jnz @ano
 xor AX,AX
@ano:
End;{keypressed}

function readkey:char; assembler;
Asm
mov AH,$10
int $16
xor AH,AH
End;{readkey}

{$endif}

{$ifdef readkey_122klaves}

function keypressed:boolean; assembler;
Asm
mov AH,$21
int $16
mov AX,1
jnz @ano
 xor AX,AX
@ano:
End;{keypressed}

function readkey:char; assembler;
Asm
mov AH,$20
int $16
xor AH,AH
End;{readkey}

{$endif}

var sw:word absolute 0:$0417;{stavovy word klavesnice}

function lshiftpressed:boolean;
Begin lshiftpressed:=(sw and 2)<>0 End;

function pshiftpressed:boolean;
Begin pshiftpressed:=(sw and 1)<>0 End;

function shiftpressed:boolean; assembler;
{Stejny princip jako vyse uvedene funkce, jenom jina forma zapisu. Asm funkce
byvaji obvykle o neco rychlejsi (a testovani shiftu pouzivam pomerne casto).}
Asm
xor AX,AX
mov ES,AX        {ES = 0}
mov SI,$0417     {SI = $0417}
mov AX,ES:[SI]   {AX = sw}
and AX,3         {kdyz vyjde nenulove cislo (stisknut libovolny Shift), je to true}
End;{shiftpressed}

function lctrlpressed:boolean;
Begin lctrlpressed:=(sw and (256 or 4))=260 End;

function pctrlpressed:boolean;
Begin pctrlpressed:=(sw and (256 or 4))=4 End;

function ctrlpressed:boolean; assembler;
Asm {Begin ctrlpressed:=(sw and 4)<>0 End;}
xor AX,AX
mov ES,AX
mov SI,$0417
mov AX,ES:[SI]
and AX,4
End;{ctrlpressed}

function laltpressed:boolean;
Begin laltpressed:=(sw and (512 or 8))=520 End;

function paltpressed:boolean;
Begin paltpressed:=(sw and (8 or 512))=8 End;

function altpressed:boolean;
Begin altpressed:=(sw and 8)=8 End;

function capslockpressed:boolean;
Begin capslockpressed:=(sw and 16384)=16384 End;

function numlockpressed:boolean;
Begin numlockpressed:=(sw and 8192)=8192 End;

function scrolllockpressed:boolean;
Begin scrolllockpressed:=(sw and 4096)=4096 End;

function capslock:boolean;
Begin capslock:=(sw and 64)=64 End;

function numlock:boolean;
Begin numlock:=(sw and 32)=32 End;

function scrolllock:boolean;
Begin scrolllock:=(sw and 16)=16 End;

procedure setcapslock(jak:boolean);
Begin
if jak then sw:=sw or 64 {nastaveni prislusneho bitu na 1...}
       else sw:=sw and 65471; {...nebo vynulovani tehoz bitu (65471=$FFFF and not 64)}
End;{setcapslock}

procedure setnumlock(jak:boolean);
Begin
if jak then sw:=sw or 32
       else sw:=sw and 65503;
End;{setnumlock}

procedure setscrolllock(jak:boolean);
Begin
if jak then sw:=sw or 16
       else sw:=sw and 65519;
End;{setscrolllock}

procedure tokeybuf(s:string); assembler;
Asm
les DI,s        {ES:DI = adresa retezce}
mov BL,[ES:DI]  {BL = delka retezce (nulty znak)}
mov AH,5        {cislo sluzby}
 @cyklus:       {pro kazdy znak:}
 inc DI         {o znak v retezci dal}
 mov CL,[ES:DI] {nacteme znak do CL}
 int $16        {zavolame preruseni klavesnice}
 dec BL     {loop nejde pouzit, protoze v CL potrebujeme mit znak pro int $16}
 jnz @cyklus    {cyklus tocime porad dokola, dokud nejsme na konci retezce}
End;{tokeybuf}

procedure RychlostKlavesnice(zpozdeni,rychlost:byte); assembler;
Asm
mov AX,$0305
mov BH,zpozdeni
mov BL,rychlost
int $16
End;{rychlostklavesnice}

const instalovano:boolean=false; {jestli je nainstalovane nove preruseni klavesnice (na zacatku samozrejme neni)}

type pole256booleanu=array[0..255] of boolean;

var PuvodniInt9:pointer; {pro ulozeni puvodniho preruseni klavesnice}
    PuvodniExit:pointer; {pro ulozeni Exitprocu}
    rozsirena:byte; {informace o tom, jestli posledni precteny scankod byl kod pro
                     rozsirenou klavesu (pak rozsirena=128) nebo ne (rozsirena=0)}
    _klavesy:^pole256booleanu;

procedure MojeInt9; interrupt; assembler;
{Pro obsluhu klavesnice, kdyz je potreba cist vic stisknutych klaves najednou.
Vola se automaticky po kazdem stisku nebo pusteni nejake klavesy.}
Asm
in AL,$60         {nacteme scankod (AL:=port[$60])}
cmp AL,$E0        {je to $E0? (rozsirena klavesa)}
jne @normalni     {neni => zpracujeme scankod}
 mov rozsirena,128  {zapamatujeme si to pro priste (teoreticky bychom sem mohli dat i jinou hodnotu nez 128)...}
 jmp @konec         {...a skoncime, skutecny scankod klavesy nam prijde za chvili v pristim preruseni}
@normalni:
mov BL,AL         {BL = AL}
and BL,$7F        {vynulujeme nejvyssi bit (v BL je ted pouze scankod)}
 and AL,$80        {v AL jeste mame hodnotu z portu, z ni pouzijeme pouze nejvyssi bit (stisknuto/pusteno)}
add BL,rozsirena  {pricteme hodnotu promenne Rozsirena, ted mame v BL skutecny index pro pole _klavesy}
 xor AL,$80        {1 znamena pusteno, ale my to chceme obracene. Timhle se zmeni 1 na 0 nebo 0 na 1.}
xor BH,BH         {budeme potrebovat cely registr BX ke spocitani adresy, tak nesmi v horni pulce nic prekazet}
 rol AL,1          {nejvyssi bit skoci na nejnizsi pozici a prevod na boolean je hotovy}
les DI,_klavesy   {ES:DI = adresa pole _klavesy}
mov ES:[DI+BX],AL {do pole na index BX (cislo klavesy) vlozime hodnotu AL (stisknuto nebo pusteno)}
mov rozsirena,0   {vynulujeme promennou Rozsirena pro priste}
@konec:
mov AL,$20
out $20,AL        {oznamime konec preruseni}
End;{mojeint9}

function pressed(klavesa:byte):boolean; assembler;
Asm     {pressed:=_klavesy^[klavesa];}
xor BX,BX        {potrebuju nulovy BH}
les DI,_klavesy
mov BL,klavesa
mov AL,ES:[DI+BX]
End;{pressed}

procedure InitKlav;
Begin
if not instalovano then
 begin
 new(_klavesy); {alokace pole pro stisknute klavesy}
 fillchar(_klavesy^,256,false); {vsechny klavesy nastav na "pusteno"}
 rozsirena:=0; {tohle bude potreba}
 getintvec(9,puvodniint9); {zaloha puvodniho preruseni}
 setintvec(9,@mojeint9); {nastaveni noveho preruseni}
 instalovano:=true;
 puvodniexit:=exitproc; {zaloha puvodni ukoncovaci procedury}
 exitproc:=@zrusklav; {presmerovani ukoncovaci procedury na odinstalovani preruseni}
 end;
End;{InitKlav}

procedure ZrusKlav; {far uz je, protoze ma hlavicku v interface}
Begin
if instalovano then
 begin
 exitproc:=puvodniexit;{vraceni puvodni ukoncovaci procedury}
 setintvec(9,puvodniint9);{nastaveni puvodniho preruseni}
 dispose(_klavesy);
 instalovano:=false;
 end;
End;{zrusklav}

function pKeyPressed:boolean; assembler;
Asm
les DI,_klavesy    {ES:DI = adresa pole _klavesy}
db $66; xor AX,AX  {EAX = 0 (tj. false v kazdem bytu)  (kdo rikal, ze DOS je 16bitovy? :-) )}
db $66; xor CX,CX  {prekladac by nesezral mov ECX,64, tak to rozdelim do dvou pulek}
mov CX,64          {ECX = 256 (pocet bytu v poli) div 4 (pocet bytu ve dwordu)}
db $66; repe scasw {repe scasd - porovnavej prvky pole s EAX tak dlouho, dokud jsou stejne a nejsi na konci pole}
or AX,CX      {= mov AX,CX (AX bylo 0)}
{Kdyz je stisknuta nejaka klavesa, cyklus repe scasd se zastavi driv nez
dojde na konec pole, a tudiz v CX zbyde nenulove cislo (=true), jinak projede
cele pole a v CX bude 0 (=false). Posledni ctyri polozky pole se tak sice
zkontrolovat nepodari, ale to nevadi, protoze tak velky scankod stejne zadna
existujici klavesa nema.}
End;{pkeypressed}

function KlavesyInstalovany:boolean;
Begin
klavesyinstalovany:=instalovano;
End;{KlavesyInstalovany}

function KlavesniceUmi(co:byte):boolean; assembler;
Asm
mov AH,9
int $16
mov CL,co    {kolikaty bit kontrolujeme}
shr AX,CL    {posuneme kontrolovany bit na nejnizsi pozici...}
and AX,1     {...a vsechny ostatni vynulujeme}
End;{klavesniceumi}

function xReadKey:word; assembler;
Asm
mov AX,$0700    {cislo sluzby pro cteni klaves}
int $21         {precteni klavesy - normalni Readkey}
or AL,AL        {neco udelame s AL, ve kterem se vratil ten znak (tak, abychom ho nezmenili), tim se nastavi ZF}
jnz @vratznak   {kdyz AL neni 0, skocime dal >> }
 int $21        {jestli jsme jeste tady, tak v AL byla 0, takze precteme dalsi znak (v AH je porad jeste $07)}
 mov AH,1       {do AH ulozime 1, aby bylo poznat, ze to byla specialni klavesa}
 jmp @konec     {a mame hotovo.}
@vratznak:
xor AH,AH     { >> vynulujeme AH, precteny znak zustane v AL a je vymalovano.}
@konec:
{vrati se obsah AX, jako u kazde funkce}
End;{xreadkey}

END.